integer FUNCTION ifsnow_fit(tw2m,t2m,sp,srh,g0,st,ol,psnow)
!
! determine whether it snowing or raining condition
!
! INPUT OPTIONS:
! (-1) For unknown input variables, assign them -999 !!! 
! (0) ol MUST BE GIVEN for any of the following options
! (1) t2m                 --> use t2m only
! (2) tw2m                --> use tw2m only
! (2') t2m+sp+srh         --> use t2m+sp+srh find tw2m first, then ...
! (3) tw2m+st             --> use tw2m and surface temp
! (3') t2m+sp+srh+st      --> find tw2m first, then use tw2m + surface temp
! (4) tw2m+g0             --> use tw2m + lapse rate
! (4') t2m+sp+srh+g0      --> find tw2m first, then use tw2m + lapse rate
! (5) tw2m+g0+st          --> complete set
! (5') t2m+sp+srh         --> find tw2m first, then ...
!
! MOST ACCURATE                 LEST ACCURATE
! 5/5' --- 4/4' --- 3/3' --- 2/2' --- 1
!
! INPUTS:
! tw2m: 2m wet-bulb temperature in C
!     -999.9 if unavailable 
! t2m: 2m temperature in C - not used if tw2m is given
! AT A MINIMUM, one of the t's (tw2m and t2m) needs to be given
! sp: surface pressure hPa - not used if tw2m is given
!     -999.9 if not available
! srh: surface relative humidity in % - not used if tw2m is given
!     -999.9 if not available
! g0: lapse rate of temperature of the lowest 0.5 km, in unit C/km 
!     g0 is positive when temperature decreases with height
!     -999.9 if not available
! st: surface skin temperature in C
!     -999.9 if not available
! ol: ocean or land, 0=ocean, else=land, must have valid input value
!
! OUTPUTS:
! ifsnow_fit:
!-1: indeterminable
! 0: Snowfall with less than 50% probability
! 1: Snowfall with over 50%, but less than 75% probability
! 2: Snowfall with over 75%, but less than 90% probability
! 3: Snowfall with over 90% probability 
! psnow:
! snow-probability (0.0-1.0)
! <0 indicates cannot be determined

use WaterVapor 
use,intrinsic :: ISO_FORTRAN_ENV

implicit none
real,parameter :: NOTAVAIL=-900
integer :: ol
real :: t2m,tw2m,g0,st,sp,srh,psnow
integer :: control=-1

if(tw2m .lt. NOTAVAIL) then  ! wet-bulb temp not available

if(t2m.gt.NOTAVAIL .and. sp.gt.NOTAVAIL .and. srh.gt.NOTAVAIL) then
        tw2m=rh2tw(sp,t2m,srh)
else if(t2m.gt.NOTAVAIL) then
        control=0
        tw2m=t2m
        goto 100
else
       write(ERROR_UNIT,*) "At A MINIMUM, Valid T2m needs to be given. T2m=",t2m
       write(ERROR_UNIT,*) "No valid snow probability cannot calculated."
       ifsnow_fit=-1
       return 
endif
endif

! wet-bulb temp available

if(g0.gt.NOTAVAIL .and. st.gt.NOTAVAIL) then
        control=4
else if(g0.gt.NOTAVAIL .and. st.le.NOTAVAIL) then
        control=3
else if(g0.le.NOTAVAIL .and. st.gt.NOTAVAIL) then
        control=2
else if(g0.le.NOTAVAIL .and. st.le.NOTAVAIL) then
        control=1
endif

100 continue
call probsnow_fit(tw2m,g0,st,ol,control,psnow)

if(psnow .ge. 0.9) then
 ifsnow_fit=3
else if(psnow .ge. 0.75) then
 ifsnow_fit=2
else if(psnow .ge. 0.5) then
 ifsnow_fit=1
else
 ifsnow_fit=0
endif
return
end

Subroutine probsnow_fit(tw,lp,surft,stype,control,psnow)
! INPUTS:
! tw: 2m wet-bulb temperature in C or 2m temperature - must have valid input value
! lp: lapse rate of temperature of the lowest 0.5 km, in unit C/km 
!     lp is positive when temperature decreases with height
!     -999.9 if not available
! surft: surface skin temperature in C, -999.9 if not available
! stype: ocean or land, 0=ocean, else=land, must have valid input value
! control: 0-----t2m known
!          1-----tw known
!          2-----tw and lp known
!          3-----tw and surft known
!          4-----tw,lp and surft known
!
! RETURN:
! psnow: snow probability (0~1)

    implicit none
    integer(4),intent(in) :: stype,control
    real(4),intent(in) :: tw,lp,surft
    real(4),intent(out) :: psnow

    !-----local vars
    real(4) :: fa,fb,ga,gb,tw2,lp2,surft2

    tw2=tw
    lp2=lp
    surft2=surft

    if( stype .eq. 0) then
        goto 1000
    else
        goto 2000
    end if

    1000 continue   ! ocean
 
    if( control .eq. 0) then ! t2m known
        if(tw2 .gt. 9.5) tw2=9.5
        if(tw2 .lt. -9.5) tw2=-9.5

        psnow= 0.013554*exp(-0.441642*(tw2-9.5))
        if(psnow .gt. 1.0) psnow=1.0     
        return
    else if( control .eq. 1) then ! tw known
        if(tw2 .gt. 9.5) tw2=9.5
        if(tw2 .lt. -9.5) tw2=-9.5

        psnow= 0.002636*exp(-0.597934*(tw2-9.5))
        if(psnow .gt. 1.0) psnow=1.0     
        return

    else if( control .eq. 2) then ! tw and lp known
        if(tw2 .gt. 7) tw2=7
        if(tw2 .lt. -7) tw2=-7
        if(lp2 .gt. 11.5) lp2=11.5
        if(lp2 .lt. -6.5) lp2=-6.5
          
        lp2=lp2+6.5
        !coefficients as function of lapse rate
        fa=(16.2869+1.0119*lp2-0.1978*lp2**2+0.0506*lp2**3)*1e-4
        fb=(-710.104-18.5849*lp2+3.3901*lp2**2-0.08895*lp2**3)*1e-3

        psnow=fa*exp(fb*(tw2-7))
        if(psnow .gt. 1.0) psnow=1.0     
        return

    else if( control .eq. 3) then ! tw and surft known
        if(tw2 .gt. 7) tw2=7
        if(tw2 .lt. -7) tw2=-7
        if(surft2 .gt. 17.0) surft2=17.0
        if(surft2 .lt. -3.0) surft2=-3.0

        surft2=surft2+3.0
        !coefficients as function of surface temperature
        fa=(31.6366-0.8601*surft2)*1e-3
        fb=(-42.9292-0.4107*surft2)*1e-2

        psnow=fa*exp(fb*(tw2-7))
        if(psnow .gt. 1.0) psnow=1.0     
        return

    else if( control .eq. 4) then ! tw, lp and surft known
        if(tw2 .gt. 6) tw2=6
        if(tw2 .lt. -6) tw2=-6
        if(lp2 .gt. 10.5) lp2=10.5
        if(lp2 .lt. -5.5) lp2=-5.5
        if(surft2 .gt. 17.0) surft2=17.0
        if(surft2 .lt. -3.0) surft2=-3.0
      
        lp2=lp2+5.5
        surft2=surft2+3.0

        !coefficients as function of lapse rate and surface temperature
        ga=(-14.9568+6.3271*lp2-2.4861*lp2**2+0.0662*lp2**3)*1e-4
        gb=(-0.2441*lp2-2.2052)*1e-2

        fa=(68.311-18.0999*lp2+7.8853*lp2**2-0.2191*lp2**3)*1e-4+ga*surft2/10.0
        fb=(-635.022+4.7558*lp2+2.8306*lp2**2-0.1252*lp2**3)*1e-3+gb*surft2/10.0

        psnow=fa*exp(fb*(tw2-6))
        if(psnow .gt. 1.0) psnow=1.0     
        return
    end if

    2000 continue ! land
    
    if( control .eq. 0) then ! t2m known
        if(tw2 .gt. 9.5) tw2=9.5
        if(tw2 .lt. -9.5) tw2=-9.5

        psnow= 0.00399*exp(-0.571375*(tw2-9.5))
        if(psnow .gt. 1.0) psnow=1.0     
    else if( control .eq. 1) then ! tw known
        if(tw2 .gt. 9.5) tw2=9.5
        if(tw2 .lt. -9.5) tw2=-9.5

        psnow= 0.0004545*exp(-0.781803*(tw2-9.5))
        if(psnow .gt. 1.0) psnow=1.0     
        return
 
    else if( control .eq. 2) then ! tw and lp known
        if(tw2 .gt. 7) tw2=7
        if(tw2 .lt. -7) tw2=-7
        if(lp2 .gt. 11.5) lp2=11.5
        if(lp2 .lt. -6.5) lp2=-6.5
   
        lp2=lp2+6.5
        !coefficients as function of lapse rate
        fa=(6.89793-1.87809*lp2+0.63496*lp2**2+0.01691*lp2**3)*1e-4
        fb=(-827.6529-9.9175*lp2+4.1869*lp2**2-0.1423*lp2**3)*1e-3

        psnow=fa*exp(fb*(tw2-7))
        if(psnow .gt. 1.0) psnow=1.0     
        return
    
    else if( control .eq. 3) then ! tw and surft known
        if(tw2 .gt. 7) tw2=7
        if(tw2 .lt. -7) tw2=-7
        if(surft2 .gt. 17.0) surft2=17.0
        if(surft2 .lt. -13.0) surft2=-13.0

        surft2=surft2+13.0
        !coefficients as function of surface temperature
        fa=(9.2197+0.5677*surft2)*1e-3
        fb=(-60.6626+0.5187*surft2)*1e-2

        psnow=fa*exp(fb*(tw2-7))
        if(psnow .gt. 1.0) psnow=1.0     
        return

    else if ( control .eq. 4) then ! tw, lp and surft known
        if(tw2 .gt. 7) tw2=7
        if(tw2 .lt. -7) tw2=-7
        if(lp2 .gt. 11.5) lp2=11.5
        if(lp2 .lt. -6.5) lp2=-6.5
        if(surft2 .gt. 17.0) surft2=17.0
        if(surft2 .lt. -13.0) surft2=-13.0

        lp2=lp2+6.5
        surft2=surft2+13.0
        !coefficients as function of lapse rate and surface temperature
        ga=(-12.8582+4.0843*lp2+0.8627*lp2**2-0.0435*lp2**3)*1e-5
        gb=(0.3772*lp2+6.9844)*1e-3

        fa=(24.2914-5.6203*lp2+1.1456*lp2**2-0.002294*lp2**3)*1e-4+ga*surft2/10.0
        fb=(-735.4863-57.4727*lp2+9.7838*lp2**2-0.3238*lp2**3)*1e-3+gb*surft2/10.0
 
        psnow=fa*exp(fb*(tw2-7))
        if(psnow .gt. 1.0) psnow=1.0     
        return

    end if

EndSubroutine
