module WaterVapor
! Module for water vapor related computations

implicit none
! constants:
real,parameter :: wv_lv=2.5E+6       ! latent heat for evapoartion (J/Kg)
real,parameter :: wv_cpd=1005.       ! specific heat w/ p=const. for dry air
real,parameter :: wv_Rv=461.5        ! water vapor gas constant                                  
!real,parameter :: wv_Rd=287.         ! Dry gas constant
real,parameter :: wv_epsln=0.622     ! Rd/Rv
real,parameter :: wv_A=2.53E+9       ! coef. in Clausius-Clapeyron eq. (mb)
real,parameter :: wv_B=5.42E+3       ! coef. in C.-C. eq. (K)
real,parameter :: wv_c2k=273.15      ! const for c to k conversion
real,parameter :: wv_SMALL=1.E-2     ! error allowed

contains

function SatVapPre(tc) result (es)
real :: es, tc
! input: tc - temperature in C
! output: es - sat. vap. pre. in mb
es = wv_A*exp(-wv_B/(tc+wv_c2k))
end function SatVapPre

function SatMixRat(pmb,tc) result (ws)
real :: ws,pmb,tc,es
! input: pmb - pressure in mb
!        tc - temperature in C
!output: ws - sat. vapor mixing ratio
es=SatVapPre(tc)
ws=wv_epsln*es/(pmb-es)
end function SatMixRat

function td2e(td) result(e)
real :: td,e
!input: td - dew point temperature in C
!output: e - vapor pressure in mb
e=SatVapPre(td)
end function td2e

function td2rh(tc,td) result (rh)
real :: tc,td,rh
!input: td - dew point temperature in C
!       tc - temperature in C
!output: rh - rel. humidity in %
rh = 100.*SatVapPre(td)/SatVapPre(tc)
end function td2rh

function td2w(pmb,td) result (w)
real :: pmb,td,w
! input: pmb - pressure in mb
!        td - dew point temperature in C
!output: w - mixing ratio (unitless)
w=SatMixRat(pmb,td)
end function td2w

function rh2e(tc,rh) result (e)
real :: tc,rh,e
!input:  tc - temperature in C
!        rh - rel. humidity in %
!output: e - vapor pressure in mb
e=0.01*rh*SatVapPre(tc)
end function rh2e

function rh2w(pmb,tc,rh) result (w)
real :: pmb,tc,rh,w,e
!input: pmb - pressure in mb
!       tc - temperature in C
!       rh - rel. humidity in %
!output:  w - mixing ratio (unitless)
e=rh2e(tc,rh)
w=wv_epsln*e/(pmb-e)
end function rh2w

function w2rh(pmb,w,tc) result (rh)
real :: pmb,tc,w,rh
rh = 100*pmb*w/wv_epsln/SatVapPre(tc)
end function w2rh

function sh2rh(sh,tc) result (rh)
!sh - vapor density in kg/m^-3
real :: tc,sh,rh
rh = 100*sh*wv_Rv*(tc+wv_c2k)*0.01/SatVapPre(tc)
end function sh2rh

function td2tw(pmb,tc,td) result(tw)
real :: pmb,tc,td,tw,rh
!input: pmb - pressure in mb
!       tc - temperature in C
!       td - dew point temperature in C
!output:tw-wet-bulb temp. in C
!w=td2w(pmb,td)
rh=td2rh(tc,td)
tw=TwByNewtonIteration(pmb,tc,rh)
end function td2tw

function rh2tw(pmb,tc,rh) result(tw)
real :: pmb,tc,rh,tw
!input: pmb - pressure in mb
!       tc - temperature in C
!       rh - rel. humidity in %
!output:tw-wet-bulb temp. in C
!w=rh2w(pmb,tc,rh)
tw=TwByNewtonIteration(pmb,tc,rh)
end function rh2tw

function TwByNewtonIteration(pmb,tc,rh) result(tw)
real :: pmb,ww,tc,tw,rh
!input: pmb - pressure in mb
!       tc - temperature in C
!       rh - rel. humidity in %
!output:tw-wet-bulb temp. in C
! w=epsln*e/(p-e)
real :: twOld,twNew,f,fprime
integer :: i,maxi=100000

twOld=tc+wv_c2k            !first guess is twOld = t
i=0
!Newton's Iteration Method:
ww=1./(pmb*exp(wv_B/(tc+wv_c2k))-wv_A)*0.01*rh              
11  f=tc+wv_c2k-twOld-wv_lv/wv_cpd*wv_epsln*wv_A*(1./(pmb*exp(wv_B/twOld)-wv_A)-ww)
    fprime=-1-wv_lv/wv_cpd*wv_epsln*wv_A*pmb*wv_B*exp(-wv_B/twOld)/twOld**2/ &
              (pmb-wv_A*exp(-wv_B/twOld))**2
    twNew=twOld-f/fprime
if(abs(twNew-twOld) .gt. wv_SMALL) then
    twOld=twNew
    i=i+1
    if(i.lt.maxi) goto 11
    print*,"Newton Iteration failed after ",maxi," loops."
    stop
else
    tw=twNew-wv_c2k ! Celsius
end if
end function TwByNewtonIteration

end module WaterVapor
